/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
/*          EXIFPROBE - TIFF/JPEG/EXIF image file probe               */
/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
/* Copyright (C) 2002 by Duane H. Hesser. All rights reserved.        */
/*                                                                    */
/* See the file LICENSE.EXIFPROBE for terms of use.                   */
/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */

#ifndef lint
static char *ModuleId = "@(#) $Id: maker_casio.c,v 1.12 2005/01/29 01:39:54 alex Exp $";
#endif

/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
/* Casio camera maker-specific routines.                              */
/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
/* Most of the information coded here is due to Eckhard Henkel        */
/* (eckhard.henkel@t-online.de) as shown by TsuruZoh Tachibanaya at:  */
/* http://www.ba.wakwak.com/~tsuruzoh/Computer/Digicams/exif-e.html   */
/* (now found at:                                                     */
/*   http://landscapeimage.com/ThumbHTML/help/exif_file_format.html)  */
/* Additional information:                                            */
/*  http://www.dicasoft.de/casiomn.htm                                */

/* EP-600 info: (added Jan 05)                                        */
/* evan@ozhiker.com                                                   */
/*    http://www.ozhiker.com/electronics/pjmt/jpeg_info/casio_mn.html */
/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */

#include <stdio.h>
#include <string.h>
#include "defs.h"
#include "extern.h"
#include "misc.h"
#include "maker.h"

/* Find the identifying number assigned to known Casio camera models. */
/* This number is used to dispatch print and interpret routines       */
/* approopriate to the current image.                                 */

int
casio_model_number(char *model,char *software)
{
    int number = GENERIC;

    if(model)
    {
        switch(*model)
        {
            case 'Q':
                if(strcmp(model,"QV-2000UX") == 0)
                    number = CASIO_QV2000UX;
                else if(strcmp(model,"QV-3000EX") == 0)
                    number = CASIO_QV3000EX;
                else if(strcmp(model,"QV-4000  ") == 0)
                    number = CASIO_QV4000;
                else if(strcmp(model,"QV-8000SX") == 0)
                    number = CASIO_QV8000SX;
                break;
            case 'E':
                if(strcmp(model,"EX-P600") == 0)
                    number = CASIO_EX_P600;
                break;
            default:
                number = GENERIC;
                break;
        }
    }
    return(number);
}

/* Dispatch a Casio print routine based upon model                    */

void
print_casio_makervalue(struct ifd_entry *entry_ptr,int make, int model,
                    char *prefix)
{

    if(entry_ptr && (PRINT_VALUE))
    {
        switch(model)
        {
            case CASIO_EX_P600:
                print_casio1_makervalue(entry_ptr,make,model,prefix);
                casio2_interpret_value(entry_ptr);
                break;
            case CASIO_QV2000UX:
            case CASIO_QV3000EX:
            case CASIO_QV4000:
            case CASIO_QV8000SX:
            case GENERIC:
            default:
                print_casio1_makervalue(entry_ptr,make,model,prefix);
                casio1_interpret_value(entry_ptr);
                break;
        }
    }
}

/* Model-specific print routine for Casio cameras. This routine is    */
/* responsible for picking off any direct entry tags which are        */
/* peculiar and will not be handled properly by print_value()         */
/* (usually UNDEFINED values which fit in the 4-byte entry value). If */
/* there are no such entries, this function simply calls              */
/* print_value().                                                     */
// ###%%% why do these name a "make" argument?

void
print_casio1_makervalue(struct ifd_entry *entry_ptr,int make, int model,
                    char *prefix)
{
    if(entry_ptr && (PRINT_VALUE))
    {
        switch(entry_ptr->tag) 
        {
            default:
                print_value(entry_ptr,prefix);
                break;
        }
    }
}


/* Dispatch a routine to decode and print offset values for Casio     */
/* cameras.                                                           */

void
print_casio_offset_makervalue(FILE *inptr,unsigned short byteorder,
    struct ifd_entry *entry_ptr,unsigned long offset_ref,char *prefix,
    int indent,int addrwidth,int make,int model,int at_offset)
{

    if(entry_ptr && (PRINT_VALUE))
    {
        switch(model)
        {
            case CASIO_EX_P600:
                casio2_offset_makervalue(inptr,byteorder,entry_ptr,
                                            offset_ref,prefix,indent,
                                            addrwidth,make,model,at_offset);
                break;
            case CASIO_QV2000UX:
            case CASIO_QV3000EX:
            case CASIO_QV4000:
            case CASIO_QV8000SX:
            default:
                casio1_offset_makervalue(inptr,byteorder,entry_ptr,
                                            offset_ref,prefix,indent,
                                            addrwidth,make,model,at_offset);
                break;
        }
    }
}

/* Model-specific routine to print UNDEFINED values found at offsets  */
/* in Casio makernotes.                                               */

void
casio1_offset_makervalue(FILE *inptr,unsigned short byteorder,
    struct ifd_entry *entry_ptr,unsigned long offset_ref,char *prefix,
    int indent,int addrwidth,int make,int model,int at_offset)
{
    char *nameoftag;
    unsigned long value_offset,current_offset;

    if(entry_ptr)
    {
        value_offset = offset_ref + entry_ptr->value;
        nameoftag = maker_tagname(entry_ptr->tag,make,model); 
        switch(entry_ptr->tag)
        {
            case 0x0e00: /* PrintIM (Epson Print Image matching)  */
                if(!(PRINT_OFFSET))
                {
                    if(PRINT_BOTH_VALUE)
                        printf("@%#lx=%lu\n",value_offset,value_offset);
                    else if(PRINT_HEX_VALUE)
                        printf("@%#lx\n",value_offset);
                    else if(PRINT_DEC_VALUE)
                        printf("@%lu\n",value_offset);
                }
                current_offset = ftell(inptr);
                if(process_pim(inptr,byteorder,entry_ptr->value,offset_ref,
                    entry_ptr->count,nameoftag,"Exif_Maker_",prefix,indent,addrwidth) == 0)
                {
                    /* At least one QV4000 image uses a PIM offset    */
                    /* relative to the entry address, even though     */
                    /* other entries in the note are TIFF-relative;   */
                    /* check for a PIM entry at that location if the  */
                    /* previous one failed. GROSS HACK!               */
                    offset_ref = current_offset - 12;
                    pushcolor(GREEN);
                    printf(" checking for entry-relative offset bug");
                    popcolor();
                    putchar('\n');
                    if(process_pim(inptr,byteorder,entry_ptr->value,offset_ref,
                                    entry_ptr->count,nameoftag,"Exif_Maker_",
                                    prefix,indent,addrwidth))
                        printred("   DARN Casio!");
                }
                break;
            default:
                print_generic_offset_makervalue(inptr,byteorder,entry_ptr,
                                        offset_ref,prefix,indent,
                                        addrwidth,make,model,at_offset);
                break;
        }
    }
}

void
casio2_offset_makervalue(FILE *inptr,unsigned short byteorder,
    struct ifd_entry *entry_ptr,unsigned long offset_ref,char *prefix,
    int indent,int addrwidth,int make,int model,int at_offset)
{
    static unsigned long jpegthumbnaillength = 0UL;
    unsigned long value_offset,current_offset;
    unsigned short marker;
    unsigned short count;
    char *nameoftag;

    if(entry_ptr)
    {
        nameoftag = maker_tagname(entry_ptr->tag,make,model); 
        value_offset = offset_ref + entry_ptr->value;
        count = entry_ptr->count;
        switch(entry_ptr->tag)
        {
            case 0x0004:    /* JpegThumbnailOffset                    */
                /* This model provides the 0x2000 tag as well as the  */
                /* 0x0003/0x0004 tags. Both entries point to the same */
                /* thumbnail, so just print values for length and     */
                /* offset tags, and display the thumbnail on the      */
                /* (self-contained) 0x2000 tag                        */
                if(!(PRINT_OFFSET))
                {
                    if(PRINT_BOTH_VALUE)
                        printf("@%#lx=%lu",value_offset,value_offset);
                    else if(PRINT_HEX_VALUE)
                        printf("@%#lx",value_offset);
                    else if(PRINT_DEC_VALUE)
                        printf("@%lu",value_offset);
                }
                break;
            case 0x2000:   /* JpegThumbnail                           */
                if(at_offset)
                {
                    print_tag_address(VALUE_AT_OFFSET,value_offset,indent,prefix,addrwidth);
                    extraindent(SUBINDENT);
                    printf("Start of JPEG Thumbnail");
                }
                if(!(PRINT_OFFSET))
                {
                    if(PRINT_BOTH_VALUE)
                        printf("@%#lx=%lu",value_offset,value_offset);
                    else if(PRINT_HEX_VALUE)
                        printf("@%#lx",value_offset);
                    else if(PRINT_DEC_VALUE)
                        printf("@%lu",value_offset);
                }
                printf(" length %ld",count);
                marker = read_imagemarker(inptr,value_offset);
                if(PRINT_SEGMENT)
                {
                    if(marker)
                    {
                        putchar('\n');
                        (void)process_jpeg_segments(inptr,value_offset,
                                                    marker,count,
                                                    indent+(SUBINDENT * 2),addrwidth);
                    }
                    print_tag_address(SEGMENT,value_offset + count - 1,
                                                indent,"-",addrwidth);
                    extraindent(SUBINDENT);
                    printf("End of JPEG Thumbnail");
                    if(marker == 0)
                        printred(" (INVALID)");
                }

                /* make certain we're at the end unless 'inptr' bad   */
                // ###%%% perhaps we shouldn't do this when the thumbnail is located past the end of the makernote?
                fseek(inptr,value_offset + entry_ptr->count,0);
                break;
            case 0x0e00: /* PrintIM (Epson Print Image matching)  */
                /* haven't seen this in Casios, but everybody who's   */
                /* using PrintIM uses this tag number, so be          */
                /* prepared.                                          */
                if(!(PRINT_OFFSET))
                {
                    if(PRINT_BOTH_VALUE)
                        printf("@%#lx=%lu\n",value_offset,value_offset);
                    else if(PRINT_HEX_VALUE)
                        printf("@%#lx\n",value_offset);
                    else if(PRINT_DEC_VALUE)
                        printf("@%lu\n",value_offset);
                }
                current_offset = ftell(inptr);
                if(process_pim(inptr,byteorder,entry_ptr->value,offset_ref,
                    entry_ptr->count,nameoftag,"Exif_Maker_",prefix,indent,addrwidth) == 0)
                {
                    /* At least one QV4000 image uses a PIM offset    */
                    /* relative to the entry address, even though     */
                    /* other entries in the note are TIFF-relative;   */
                    /* check for a PIM entry at that location if the  */
                    /* previous one failed. GROSS HACK!               */
                    offset_ref = current_offset - 12;
                    pushcolor(GREEN);
                    /* We've already seen an error message            */
                    printf(" checking for entry-relative offset bug");
                    popcolor();
                    putchar('\n');
                    if(process_pim(inptr,byteorder,entry_ptr->value,offset_ref,
                                    entry_ptr->count,nameoftag,"Exif_Maker_",
                                    prefix,indent,addrwidth))
                        printred("   DARN Casio!");
                }
                break;
            default:
                print_generic_offset_makervalue(inptr,byteorder,entry_ptr,
                                        offset_ref,prefix,indent,
                                        addrwidth,make,model,at_offset);
                break;
        }
    }
}


/* Casio-specific tagnames for makernotes.                            */

char *
maker_casio_tagname(unsigned short tag,int model)
{
    char *tagname = (char *)0;

    /* Check specific models first                                    */
    switch(model)
    {
        case CASIO_QV2000UX:
        case CASIO_QV3000EX:
        case CASIO_QV8000SX:
            tagname = maker_casio1_tagname(tag,model);
            break;
        case CASIO_EX_P600:
            tagname = maker_casio2_tagname(tag,model);
            break;
        /* Add special-case models here                               */
        default:
            tagname = maker_casio1_tagname(tag,model);
            break;
    }

    /* If no model-specific tag is found, use a "generic" Casio tag   */
    if(tagname == NULL)
    {
        switch(tag)
        {
            case 0x0e00: tagname = "PrintIM"; break;
            default: break;
        }
    }
    return(tagname);
}


char *
maker_casio1_tagname(unsigned short tag,int model)
{
    char *tagname = (char *)0;

    switch(tag)
    {
        case 0x0001: tagname = "RecordingMode"; break;
        case 0x0002: tagname = "Quality"; break;
        case 0x0003: tagname = "FocusingMode"; break;
        case 0x0004: tagname = "FlashMode"; break;
        case 0x0005: tagname = "FlashIntensity"; break;
        case 0x0006: tagname = "ObjectDistance"; break;
        case 0x0007: tagname = "WhiteBalance"; break;
        case 0x000a: tagname = "DigitalZoom"; break;
        case 0x000b: tagname = "Sharpness"; break;
        case 0x000c: tagname = "Contrast"; break;
        case 0x000d: tagname = "Saturation"; break;
        case 0x0014: tagname = "ISO_CCDSensitivity"; break;
        default: break;
    }
    return(tagname);
}


char *
maker_casio2_tagname(unsigned short tag,int model)
{
    char *tagname = (char *)0;

    switch(tag)
    {
        case 0x0002: tagname = "JpegThumbnailDimensions"; break;
        case 0x0003: tagname = "JpegThumbnailLength"; break;
        case 0x0004: tagname = "JpegThumbnailOffset"; break;
        case 0x0008: tagname = "QualityMode"; break;
        case 0x0009: tagname = "ImageSize"; break;
        case 0x000D: tagname = "FocusMode"; break;
        case 0x0014: tagname = "IsoSensitivity"; break;
        case 0x0019: tagname = "WhiteBalance"; break;
        case 0x001D: tagname = "FocalLength"; break; /* units .1mm   */
        case 0x001F: tagname = "Saturation"; break;
        case 0x0020: tagname = "Contrast"; break;
        case 0x0021: tagname = "Sharpness"; break;
        case 0x2000: tagname = "JpegThumbnail"; break;
        case 0x2011: tagname = "WhiteBalanceBias"; break;
        case 0x2012: tagname = "WhiteBalance"; break;
        case 0x2022: tagname = "ObjectDistance"; break;
        case 0x2034: tagname = "FlashDistance"; break;
        case 0x3000: tagname = "RecordMode"; break;
        case 0x3001: tagname = "SelfTimer?"; break;
        case 0x3002: tagname = "Quality"; break;
        case 0x3003: tagname = "FocusMode"; break;
        case 0x3006: tagname = "TimeZone"; break;
        case 0x3007: tagname = "BestshotMode"; break;
        case 0x3014: tagname = "CCD_ISO_Sensitivity"; break;
        case 0x3015: tagname = "ColorMode"; break;
        case 0x3016: tagname = "Enhancement"; break;
        case 0x3017: tagname = "Filter"; break;
        default: break;
    }
    return(tagname);
}

/* Report the "meaning" of tag values found directly in a Casio       */
/* MakerNote IFD entry (not at an offset).                            */

void
casio1_interpret_value(struct ifd_entry *entry_ptr)
{
    switch(entry_ptr->tag)
    {
        case 0x0001:    /* RecordingMode */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 1: printf("Single Shutter"); break;
                case 2: printf("Panorama"); break;
                case 3: printf("Night Scene"); break;
                case 4: printf("Portrait"); break;
                case 5: printf("Landscape"); break;
                default: printred("undefined"); break;
            }
            print_endvalue();
            break;
        case 0x0002:    /* Quality */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 1: printf("Economy"); break;
                case 2: printf("Normal"); break;
                case 3: printf("Fine"); break;
                default: printred("undefined"); break;
            }
            print_endvalue();
            break;
        case 0x0003:    /* Focusing Mode */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 2: printf("Macro"); break;
                case 3: printf("Auto"); break;
                case 4: printf("Manual"); break;
                case 5: printf("Infinity"); break;
                default: printred("undefined"); break;
            }
            print_endvalue();
            break;
        case 0x0004:    /* Flash Mode */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 1: printf("Auto"); break;
                case 2: printf("On"); break;
                case 3: printf("Off"); break;
                case 4: printf("Redeye"); break;
                default: printred("undefined"); break;
            }
            print_endvalue();
            break;
        case 0x0005:    /* Flash Intensity */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 11: printf("Weak"); break;
                case 13: printf("Normal"); break;
                case 15: printf("Strong"); break;
                default: printred("undefined"); }
            break;
        case 0x0006:    /* Object Distance */
            pushcolor(UNCERTAIN_COLOR);
            printf(" mm");
            popcolor();
            break;
        case 0x0007:    /* White Balance */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 1: printf("Auto"); break;
                case 2: printf("Tungsten"); break;
                case 3: printf("Daylight"); break;
                case 4: printf("Fluorescent"); break;
                case 5: printf("Shade"); break;
                case 129: printf("Manual"); break;
                default: printred("undefined"); break;
            }
            print_endvalue();
            break;
        case 0x000a:    /* Digital Zoom */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 0x10000: printf("Off"); break;
                case 0x10001: printf("On"); break;
                default: printred("undefined"); break;
            }
            print_endvalue();
            break;
        case 0x000b:    /* Sharpness */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 0: printf("Normal"); break;
                case 1: printf("Soft"); break;
                case 2: printf("Hard"); break;
                default: printred("undefined"); break;
            }
            print_endvalue();
            break;
        case 0x000c:    /* Contrast */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 0: printf("Normal"); break;
                case 1: printf("Low"); break;
                case 2: printf("High"); break;
                default: printred("undefined"); break;
            }
            print_endvalue();
            break;
        case 0x000d:    /* Saturation */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 0: printf("Normal"); break;
                case 1: printf("Low"); break;
                case 2: printf("High"); break;
                default: printred("undefined"); break;
            }
            print_endvalue();
            break;
        default:
            break;
    }
}

void
casio2_interpret_value(struct ifd_entry *entry_ptr)
{
    char *tagname = (char *)0;

    switch(entry_ptr->tag)
    {
        case 0x0002: /* ThumbnailDimensions" */
            printf(" pixels");
            break;
        case 0x0008: /* QualityMode" */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 1: printf("Fine"); break;
                case 2: printf("Super Fine"); break;
                default: printf("unknown"); break;
            }
            print_endvalue();
            break;
        case 0x0009: /* ImageSize" */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 1: printf("640x480"); break;
                case 4: printf("1600x1200"); break;
                case 5: printf("2048x1536"); break;
                case 20: printf("2288x1712"); break;
                case 21: printf("2592x1944"); break;
                case 22: printf("2304x1728"); break;
                case 36: printf("3008x2008"); break;
                default: printf("unknown"); break;
            }
            print_endvalue();
            break;
        case 0x000D: /* FocusMode" */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 1: printf("Normal"); break;
                case 2: printf("Macro"); break;
                default: printf("unknown"); break;
            }
            print_endvalue();
            break;
        case 0x0014: /* IsoSensitivity" */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 3: printf("50"); break;
                case 4: printf("64"); break;
                case 6: printf("100"); break;
                case 9: printf("200"); break;
                default: printf("unknown"); break;
            }
            print_endvalue();
            break;
        case 0x0019: /* WhiteBalance" */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 1: printf("Fine"); break;
                case 2: printf("Shade"); break;
                case 3: printf("Tungsten"); break;
                case 4: printf("Fluorescent"); break;
                case 5: printf("Manual"); break;
                default: printf("unknown"); break;
            }
            print_endvalue();
            break;
        case 0x001D: /* FocalLength"; break; /* units .1mm   */
        case 0x001F: /* Saturation" */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 0: printf("-1"); break;
                case 1: printf("Normal"); break;
                case 2: printf("+1"); break;
                default: printf("unknown"); break;
            }
            print_endvalue();
            break;
        case 0x0020: /* Contrast" */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 0: printf("-1"); break;
                case 1: printf("Normal"); break;
                case 2: printf("+1"); break;
                default: printf("unknown"); break;
            }
            print_endvalue();
            break;
        case 0x0021: /* Sharpness" */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 0: printf("-1"); break;
                case 1: printf("Normal"); break;
                case 2: printf("+1"); break;
                default: printf("unknown"); break;
            }
            print_endvalue();
            break;
        case 0x2012: /* WhiteBalance" */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 0: printf("Manual"); break;
                case 1: printf("Auto"); break;
                case 4: printf("Flash"); break;
                case 12: printf("Flash"); break;
                default: printf("unknown"); break;
            }
            print_endvalue();
            break;
        case 0x2022: /* ObjectDistance" */
            printf("mm");
            break;
        case 0x2034: /* FlashDistance" */
#if 0
Off?
            if(entry_ptr->value == 0)
            {
                print_startvalue();
                printf("Off");
                print_endvalue();
            }
#endif
            break;
        case 0x3000: /* RecordMode" */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 2: printf("Normal"); break;
                default: printf("unknown"); break;
            }
            print_endvalue();
            break;
        case 0x3001: /* SelfTimer?" */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 1: printf("Off"); break;
                default: printf("unknown"); break;
            }
            print_endvalue();
            break;
        case 0x3002: /* Quality" */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 3: printf("Fine"); break;
                default: printf("unknown"); break;
            }
            print_endvalue();
            break;
        case 0x3003: /* FocusMode" */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 1: printf("Fixed"); break;
                case 6: printf("Multi-Area Auto"); break;
                default: printf("unknown"); break;
            }
            print_endvalue();
            break;
        case 0x3007: /* BestshotMode" */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 0: printf("Off"); break;
                default: printf("unknown"); break;
            }
            print_endvalue();
            break;
        case 0x3015: /* ColorMode" */
#if 0
Off???
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 0: printf("Off"); break;
                default: printf("unknown"); break;
            }
            print_endvalue();
#endif
            break;
        case 0x3016: /* Enhancement" */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 0: printf("Off"); break;
                default: printf("unknown"); break;
            }
            print_endvalue();
            break;
        case 0x3017: /* Filter" */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 0: printf("Off"); break;
                default: printf("unknown"); break;
            }
            print_endvalue();
            break;
        default: break;
    }
}
/* Some makernotes record offsets as LONG values rather than simply   */
/* using UNDEFINED with a size; the same technique used for           */
/* the ExifIFDPointer itself, or JpegInterchangeFormat, etc.          */
/* This routine hardcodes such tags so that the print_makerentry()    */
/* routine will present them as offsets.                              */

/* Asahi has the unfortunate characteristic that tag numbers are      */
/* re-used for different purposes in the second version, and one of   */
/* those tags is an offset masquerading as a long.                    */

int
maker_casio_is_long_offset(struct ifd_entry *entry_ptr,int model)
{
    int is_offset = 0;

    switch(model)
    {
        case CASIO_EX_P600:
            if(entry_ptr)
            {
                switch(entry_ptr->tag)
                {
#if 0
                    case 0x0003: is_offset = -1; break;
#endif
                    case 0x0004: is_offset = 1; break; /* must        */
                                                       /* trigger     */
                                                       /* offset      */
                                                       /* processing  */
                    default: break;
                }
            }
            break;
        default:
            break;
    }

    return(is_offset);
}
