/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
/*          EXIFPROBE - TIFF/JPEG/EXIF image file probe               */
/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
/* Copyright (C) 2002 by Duane H. Hesser. All rights reserved.        */
/*                                                                    */
/* See the file LICENSE.EXIFPROBE for terms of use.                   */
/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */

#ifndef lint
static char *ModuleId = "@(#) $Id: readfile.c,v 1.20 2005/01/27 01:56:07 alex Exp $";
#endif

/* This file contains routines for reading specific value types with  */
/* byte-swapping for integer values, specialized routines for         */
/* strings, and a general IFD entry reader.                           */

/* The code needs to know the native byteorder of the machine, which  */
/* is currently not autoconfigured. If not sure, look for a file like */
/* <sys/endian.h>. "MOTOROLA" byteorder is "big-endian", "INTEL"      */
/* byteorder is "little-endian". The code does not currently handle   */
/* "pdp11" byteorder.                                                 */


#include <stdio.h>
#include <sys/types.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include "byteorder.h"
#include "defs.h"
#include "extern.h"
#include "misc.h"
#include "tags.h"


/* Read a short integer value in the expectation that it is a TIFF or */
/* JPEG header marker/tag. If the value is a TIFF indicator, the      */
/* magic number is checked and a warning issued if invalid (the       */
/* process will fail later if necessary).                             */

/* The return is the value read (which will be a valid byte-order     */
/* indicator if the file is a valid TIFF file)                        */

unsigned short
read_imagemarker(FILE *inptr,unsigned long offset)
{
    unsigned short marker,magic;

    marker = 0;
    if(inptr)
    {
        /* The order of bytes for this read does not matter for TIFF, */
        /* but must be high-byte first for JPEG.                      */
        marker = read_ushort(inptr,TIFF_MOTOROLA,offset);
        switch(marker)
        {
            case TIFF_MOTOROLA:
            case TIFF_INTEL:
                if((magic = read_ushort(inptr,marker,HERE)) != TIFF_MAGIC)
                {
                    printf(" invalid TIFF magic (%#x)",magic);
                    printf(", apparent TIFF byteorder is %s (%#x)\n",
                            tagname(marker),marker);
                }
                break;
            case JPEG_SOI:
                break;
            default:
                break;
        }
    }
    return(marker);
}

/* Read a generic TIFF IFD entry at the given offset, swapping        */
/* byteorder of entry values as required.                             */

/* The return is a pointer to a static structure representing the     */
/* entry.                                                             */

struct ifd_entry *
read_entry(FILE *inptr,unsigned short byteorder,unsigned long offset)
{
    static struct ifd_entry entry;
    unsigned char *value;

    memset(&entry,0,sizeof(struct ifd_entry));
    if(inptr)
    {
        entry.tag = read_ushort(inptr,byteorder,offset);
        entry.value_type = read_ushort(inptr,byteorder,HERE);
        entry.count = read_ulong(inptr,byteorder,HERE);
        value = read_bytes(inptr,4,HERE);
        if(value)
        {
            if((value_type_size(entry.value_type) * entry.count) > 4)
                entry.value = to_ulong(value,byteorder);    /* offset */
            else    /* actual value(s)                                */
            {
                entry.value = 0L;
                switch(entry.value_type)
                {
                    case SSHORT: 
                    case SHORT:
                        if(entry.count == 1)
                            entry.value = to_ushort(value,byteorder);
                        else
                        {
                            entry.value = to_ushort(value,byteorder);
                            if(byteorder == TIFF_MOTOROLA)
                                entry.value |= (((unsigned long)to_ushort(value + 2,byteorder) & 0xffff) << 16);
                            else
                                entry.value = (to_ushort(value + 2,byteorder) & 0xffff) |
                                    ((entry.value & 0xffff) << 16);
                        }
                        break;
                    case BYTE:
                    case SBYTE:
                    case UNDEFINED:
                    case ASCII:
                        entry.value = *(unsigned long *)value;
                        break;
                    case LONG:
                    case SLONG:
                    default:
                        entry.value = to_ulong(value,byteorder);
                        break;
                }
            }
        }
        else
            entry.value = 0L;
    }
    return(&entry);
}

/* Read 2 bytes at the given offset and interpret them as an unsigned */
/* short integer value, swapping bytes as necessary.                  */

unsigned short
read_ushort(FILE *inptr,unsigned short byteorder,unsigned long offset)
{
    unsigned char rbuf[2];
    unsigned short value = 0;

    if(inptr)
    {
        if((offset != HERE) && (fseek(inptr,(long)offset,0) == -1))
        {
            printred(" SEEK FAILED to read unsigned short at offset ");
            printf("%lu",offset);
        }
        else if(fread(rbuf,1,sizeof(unsigned short),inptr) == sizeof(unsigned short))
            value = to_ushort(rbuf,byteorder);
        else
        {
            printred(" FAILED to read unsigned short value at offset ");
            printf("%lu",offset);
        }
        if(ferror(inptr))
            printred(strerror(errno)),putchar('\n');
        else if(feof(inptr))
            printred(" (EOF)\n");
    }
    return(value);
}

/* read a single byte at the given offset and return it as an         */
/* unsigned short value                                               */

unsigned short
read_ubyte(FILE *inptr,unsigned long offset)
{
    unsigned char rbuf[1];
    unsigned short value = 0;

    if(inptr)
    {
        if((offset != HERE) && (fseek(inptr,(long)offset,0) == -1))
        {
            printred(" SEEK FAILED to read unsigned byte at offset ");
            printf("%lu",offset);
        }
        else if(fread(rbuf,1,1,inptr) == 1)
            value = rbuf[0] & 0xff;
        else
        {
            printred(" FAILED to read unsigned byte at offset ");
            printf("%lu",offset);
        }
        if(ferror(inptr))
            printred(strerror(errno));
        else if(feof(inptr))
            printred(" (EOF)\n");
    }
    return(value);
}

/* Read 4 bytes at the given offset and interpret as an unsigned long */
/* integer, swapping bytes as necessary                               */

unsigned long
read_ulong(FILE *inptr,unsigned short byteorder,unsigned long offset)
{
    unsigned char rbuf[4];
    unsigned long value = 0;

    if(inptr)
    {
        if((offset != HERE) && (fseek(inptr,(long)offset,0) == -1))
        {
            printred(" SEEK FAILED to read unsigned 32bit integer at offset ");
            printf("%lu",offset);
        }
        else if(fread(rbuf,1,4,inptr) == 4)
            value = to_ulong(rbuf,byteorder);
        else
        {
            printred(" FAILED to read unsigned 32bit integer at offset ");
            printf("%lu",offset);
        }
        if(ferror(inptr))
            printred(strerror(errno));
        else if(feof(inptr))
            printred(" (EOF)\n");
    }
    return(value);
}

/* Read 4 bytes at the given offset and interpret as a 'float',       */
/* swapping bytes as necessary.                                       */

float
read_float(FILE *inptr,unsigned short byteorder,unsigned long offset)
{
    unsigned char rbuf[sizeof(float)];
    float value = 0.0;

    if(inptr)
    {
        if((offset != HERE) && (fseek(inptr,(long)offset,0) == -1))
            printf(" seek failed to offset %lu to read unsigned long\n",
                    offset);
        else if(fread(rbuf,1,sizeof(float),inptr) == sizeof(float))
            value = to_float(rbuf,byteorder);
        else
            printf(" failed to read float value at offset %lu\n",offset);
    }
    return(value);
}

/* Read 8 bytes at the given offset and interpret as a double,        */
/* swapping bytes as necessary.                                       */

double
read_double(FILE *inptr,unsigned short byteorder,unsigned long offset)
{
    unsigned char rbuf[sizeof(double)];
    double value = 0.0;

    if(inptr)
    {
        if((offset != HERE) && (fseek(inptr,(long)offset,0) == -1))
            printf(" seek failed to offset %lu to read unsigned long\n",
                    offset);
        else if(fread(rbuf,1,sizeof(double),inptr) == sizeof(double))
            value = to_double(rbuf,byteorder);
        else
            printf(" failed to read double value at offset %lu\n",offset);
    }
    return(value);
}

/* Read 'nbytes' of bytes at the given offset; no byte swapping.      */
/* The result is a chunk of bytes...not a string.                     */

unsigned char *
read_bytes(FILE *inptr,unsigned long nbytes,unsigned long offset)
{
    static unsigned char rbuf[MAXBUFLEN];
    unsigned char *value = (unsigned char *)0;

    rbuf[0] = '\0';
    if(nbytes > MAXBUFLEN)
    {
        printf(" read %lu bytes would overrun buffer; truncating to %d\n",
                nbytes,MAXBUFLEN);
        nbytes = MAXBUFLEN;
    }
    if(nbytes)
    {
        if(inptr)
        {
            if((offset != HERE) && (fseek(inptr,(long)offset,0) == -1))
            {
                printred(" SEEK FAILED to read unsigned bytes at offset ");
                printf("%lu",offset);
            }
            else
            {
                memset(rbuf,0,MAXBUFLEN);
                if(fread(rbuf,1,nbytes,inptr) == nbytes)
                    value = rbuf;
                else
                {
                    printred(" FAILED to read unsigned byte at offset ");
                    printf("%lu",offset);
                }
            }
            if(ferror(inptr))
                printred(strerror(errno));
            else if(feof(inptr))
                printred(" (EOF)");
        }
    }
    return(value);
}

/* convert a couple of bytes to an unsigned short integer in native   */
/* byte order, swapping bytes if file byteorder differs.              */
/* If byteorder is not TIFF_INTEL or TIFF_MOTOROLA, the native byte   */
/* order is used.                                                     */

unsigned short
to_ushort(unsigned char *buf,unsigned short byteorder)
{
    unsigned short value = 0;

    if(buf)
    {
        value = *(unsigned short *)buf;
#ifdef NATIVE_BYTEORDER_BIGENDIAN
        if(byteorder == TIFF_INTEL)
            value = (buf[1] << 8) | buf[0];
#else
        if(byteorder == TIFF_MOTOROLA)
            value = (buf[0] << 8) | buf[1];
#endif
    }
    return(value);
}

/* convert a bunch of bytes to an unsigned long integer in native     */
/* byte order, swapping bytes if file byteorder differs.              */
/* If byteorder is not TIFF_INTEL or TIFF_MOTOROLA, the native byte   */
/* order is used.                                                     */

unsigned long
to_ulong(unsigned char *buf,unsigned short byteorder)
{
    u_int32_t value = 0;

    if(buf)
    {
        value = *(u_int32_t *)buf;
#ifdef NATIVE_BYTEORDER_BIGENDIAN
        if(byteorder == TIFF_INTEL)
            value = (buf[3] << 24) | (buf[2] << 16) | (buf[2] << 8) | buf[0];
#else
        if(byteorder == TIFF_MOTOROLA)
            value = (buf[0] << 24) | (buf[1] << 16) | (buf[2] << 8) | buf[3];
#endif
    }
    return((unsigned long)value);
}

/* convert a bunch of bytes to a 'float' value in native byte order,  */
/* swapping bytes if file byteorder differs. If byteorder is not      */
/* TIFF_INTEL or TIFF_MOTOROLA, the native byte order is used.        */

/* This is pretty primitive; it assumes that the native floating      */
/* point format is the same as the floating point format written to   */
/* the image file, probably IEEE as required by the TIFF6             */
/* specification. See 'libtiff' for some conversion routines.         */

float
to_float(unsigned char *buf,unsigned short byteorder)
{
    float value = 0.0;
    unsigned char swap[sizeof(float)];
    unsigned char *p;
    int i,size;


    if(buf)
    {
        p = buf;
#ifdef NATIVE_BYTEORDER_BIGENDIAN
        if(byteorder == TIFF_INTEL)
#else
        if(byteorder == TIFF_MOTOROLA)
#endif
        {
            size = sizeof(float);
            for(i = 0; i < size; ++i)
                swap[i] = buf[size - 1 - i];
            p = swap;
        }
        value = *(float *)p;
    }
    return(value);
}

/* convert a bunch of bytes to a 'double' value in native byte order, */
/* swapping bytes if file byteorder differs. If byteorder is not      */
/* TIFF_INTEL or TIFF_MOTOROLA, the native byte order is used.        */

/* This is pretty primitive; it assumes that the native floating      */
/* point format is the same as the floating point format written to   */
/* the image file, probably IEEE as required by the TIFF6             */
/* specification. See 'libtiff' for some conversion routines.         */

double
to_double(unsigned char *buf,unsigned short byteorder)
{
    double value = 0.0;
    unsigned char swap[sizeof(double)];
    unsigned char *p;
    int i,size;


    if(buf)
    {
        p = buf;
#ifdef NATIVE_BYTEORDER_BIGENDIAN
        if(byteorder == TIFF_INTEL)
#else
        if(byteorder == TIFF_MOTOROLA)
#endif
        {
            size = sizeof(double);
            for(i = 0; i < size; ++i)
                swap[i] = buf[size - 1 - i];
            p = swap;
        }
        value = *(double *)p;
    }
    return(value);
}

/* Read a few bytes at 'offset' which are expected to be a JPEG APPn  */
/* id string, where 'n' is given as an argument.                      */

char *
read_appstring(FILE *inptr,unsigned short apptype,unsigned long offset)
{
    unsigned long length;
    char *appstring = (char *)0;


    if(inptr)
    {
        switch(apptype)
        {
            case JPEG_APP0:
                length = 32;    /* should be 5; allow for garbage     */
                break;
            case JPEG_APP1:
            case JPEG_APP2:
                length = 32;    /* should be 5; allow for garbage     */
                break;
            default:
                length = 32; 
                break;
        }
        if(length)
            appstring = read_string(inptr,offset,length);
    }
    return(appstring);
}

/* Read an ascii string, up to a null byte or 'max_bytes', at         */
/* 'offset' relative to the beginning of the file.                    */
/* Returns the string in static storage.  Use it or lose it.          */

char *
read_string(FILE *inptr,unsigned long offset,unsigned long max_bytes)
{
    static char stringbuf[MAXBUFLEN];
    char *bufp;
    int nread = 0;

    bufp = stringbuf;
    *bufp = '\0';

    if(inptr)
    {
        if((offset != HERE) && (fseek(inptr,(long)offset,0) == -1))
        {
            printred(" SEEK FAILED to read ascii string at offset ");
            printf("%lu",offset);
        }
        else if(max_bytes > 0)
        {
            if(max_bytes > MAXBUFLEN)
                max_bytes = MAXBUFLEN - 2;
            while((nread < max_bytes) &&
                    (*bufp = fgetc(inptr)) &&
                        !feof(inptr) && 
                            !ferror(inptr) &&
                                    isascii(*bufp++))
            {
                    ++nread;
            }
            *bufp = '\0';
        }
    }
    return(stringbuf);
}
